#
# Copyright (c) 2020 by Philipp Scheer. All Rights Reserved.
#

from urllib import request, parse
from multiprocessing import Process
import json, time


class Jarvis:
	def __init__(self, host, token, port=2021):
		self.host = host
		self.token = token
		self.port = port
		self.connected = False


	
	def connect(self, reconnect_on_error=True):
		resp = self._post("register-device?name={}&token={}&type={}&native={}".format("Jarvis Headset", self.token, "headset", "true"), {})
		resp = json.loads(resp)
		if resp["success"]:
			self.connected = True
			self._start_hello_thread()
			return True
		else:
			self.connected = False
			if reconnect_on_error:
				return self.reconnect()
			return False
	def reconnect(self):
		resp = self._post(	"am-i-registered?token={}".format(self.token), {})
		resp = json.loads(resp)
		if resp["success"]:
			self.connected = True
			self._start_hello_thread()
			return True
		else:
			self.connected = False
			return False



	def set_property(self, key, value):
		pass



	def _start_hello_thread(self):
		self.hello_process = Process(target=Jarvis._hello_thread, args=(self.host, self.token, self.port))
		self.hello_process.start()
	def _stop_hello_thread(self):
		self.hello_process.terminate()

	def _post(self, endpoint, post_data):
		url = "http://{}:{}/{}".format(self.host, self.port, endpoint) 
		data = str(json.dumps(post_data)).encode('utf-8')
		req = request.Request(url, data=data)
		return request.urlopen(req).read()

	@staticmethod
	def _hello_thread(h,t,p):
		while True:
			req = request.Request("http://{}:{}/{}".format(h,p,"hello?token="+t))
			req.read()
			time.sleep(5)


#
# Copyright (c) 2020 by Philipp Scheer. All Rights Reserved.
#

import paho.mqtt.client as mqtt
import random, string

# MQTT(host=127.0.0.1, port=1883, client_id=[random])
# 	.on_connect(callback[client, userdata, flags, rc])
# 	.on_message(callback[client, userdata, message])
# 	.publish(topic, payload)
# 	.subscribe(topic)
class MQTT():
	def __init__(self, host="127.0.0.1", port=1883, client_id=None):
		self.host = host
		self.port = port

		if client_id is None:
			self.client_id = ''.join(random.choices(string.ascii_uppercase + string.digits, k=8))
		else:
			self.client_id = str(client_id)
		
		self.client = mqtt.Client(client_id=client_id)
		self.client.connect(self.host, self.port)
		self.client.loop_start()

	def on_connect(self, fn):
		self.client.on_connect = fn

	def on_message(self, fn):
		self.client.on_message = fn

	def publish(self, topic, payload, disable_log=False):
		return self.client.publish(topic, payload)

	def subscribe(self, topic):
		return self.client.subscribe(topic)




#
# Copyright (c) 2020 by Philipp Scheer. All Rights Reserved.
#

from datetime import datetime
import os, zipfile, json

#MAX_LOGF_SIZE = 100 * 1024 * 1024		# 100MB
MAX_LOGF_SIZE = 20 * 1024 * 1024		# 20MB
MAX_FAST_LENGTH = 1000					# 1000 entries

class Logger:
	def __init__(self, logfile, compressed_folder):
		self.logfile = logfile
		self.compressed_folder = compressed_folder
		self.on = True
		self.fast = False
		self.grouping = False
		self.to_console = False
		self.fast_log_data = []

	def off(self):
		self.on = False
	def on(self):
		self.on = True

	def console_on(self):
		self.to_console = True
	def console_off(self):
		self.to_console = False

	def new_group(self, data):
		self.grouping = True
		self.fast_log_data.append({"meta":data, "data":[]})

	def i(self, tag, message):
		self.wr("I", tag, message)
	def e(self, tag, message):
		self.wr("E", tag, message)
	def w(self, tag, message):
		self.wr("W", tag, message)
	def s(self, tag, message):
		self.wr("S", tag, message)
	def c(self, tag, message):
		self.wr("C", tag, message)


	def enable_fast(self):
		self.fast = True
		self.clear_fast()
	def disable_fast(self):
		self.fast = False
		self.clear_fast()
	def clear_fast(self):
		del self.fast_log_data
		self.fast_log_data = []
	def get_fast(self):
		return self.fast_log_data

	def wr(self, pre, tag, message):
		if self.fast:
			if len(self.fast_log_data) > MAX_FAST_LENGTH:
				self.clear_fast()
			
			msg_to_store = message # should be an object
			if isinstance(message, str):
				try:
					msg_to_store = json.loads(message)
				except Exception as e:
					# print(e)
					pass

			if self.grouping:
				# print(len(self.fast_log_data))
				self.fast_log_data[-1]["data"].append({
					"severity": pre,
					"tag": tag,
					"message": msg_to_store
				})
			if not self.grouping:
				self.fast_log_data.append({
					"severity": pre,
					"tag": tag,
					"message": msg_to_store
				})

		logstr = "{} {}/{}{} {}".format(str(datetime.now()), pre, tag, " " * (10-len(tag)), message)
		
		if self.to_console:
			print(logstr)
		if not self.on:
			return

		with open(self.logfile, "a+") as f:
			f.write(logstr + "\n")
		self.check_file_size()
	
	def check_file_size(self):
		if os.path.getsize(self.logfile) > MAX_LOGF_SIZE:
			# TODO: replace logs/... with consistent path
			zipfile.ZipFile("{}/jarvis_http.log.{}.zip".format(self.compressed_folder, str(datetime.now()).replace(" ", "_").replace(":", "-")), 'w', zipfile.ZIP_DEFLATED).write(self.logfile)
			with open(self.logfile, "w") as f:
				f.write("")
		